# -*- coding: utf-8 -*-

"""
Copyright(C) 2007-2008 INL
Written by Romain Bignon <romain AT inl.fr>
           Pierre Chifflier <chifflier AT inl.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

$Id$
"""

from twisted.enterprise import adbapi # SQL
from logging import info, debug

def tr(msg): return msg

class DatabaseError(Exception):
    pass

#############################################
#                                           #
#            Database Object                #
#                                           #
#############################################
class GenericDataBase:

    types = {}

    dbtype = None
    dbapi = None
    kw_max_limit = ""

    dbkw = { 'host' : 'host', 'db': 'db', 'user' : 'user', 'pass': 'passwd' }

    error_type = None

    def __init__(self, conf):
        """
            Build DataBase object.
        """
        self.dbpool = None
        self.conf = conf
        self.ip_type = 4
        adbapi.ConnectionPool.noisy = False
        adbapi.ConnectionPool.reconnect = True

    def start(self):
        """
            Create a connection to database.
        """

        if self.dbpool:
            self.dbpool.close()

        self.dbpool = None

        try:
            info('Connecting to database %s' % self.conf['db'])
            kwargs = {}
            kwargs[ self.dbkw['host'] ] = self.conf['host']
            kwargs[ self.dbkw['db'] ]   = self.conf['db']
            kwargs[ self.dbkw['user'] ] = self.conf['user']
            kwargs[ self.dbkw['pass'] ] = self.conf['password']

            self.dbpool = adbapi.ConnectionPool(self.dbapi, **kwargs)
            self.dbpool.connect()
            info('Connected!')
        except self.error_type, e:
            self.dbpool = None
            if hasattr(e, '__iter__'):
                e = e[1]
            raise DatabaseError(tr("Unable to connect to database:\n%s") % unicode(e))

    def close(self):

        self.dbpool.close()

    def createRequest(self, table=None):

        if self.types.has_key(self._type):
            obj = self.types[self._type]
        else:
            raise DatabaseError("Unknown database type %s" % self._type)

        if not table:
            table = self.ulog

        return obj(self, table, self.ip_type)

    def query_error(self, error, method=None):

        if error.type in (self.error_type,adbapi.ConnectionLost):
            if self.dbpool is not None and method is not None:
                info("Retrying NuLog database query...")
                self.start()
                return method().addErrback(self.query_error, method)
            else:
                self.dbpool = None
                raise DatabaseError('Connection lost to database: %s' % error)
        else:
            error.raiseException()

    def query(self, query):

        def cb(value):
            return (value, len(value))

        debug('Query: %s' % query)
        return self.dbpool.runQuery(query).addErrback(self.query_error, lambda: self.dbpool.runQuery(query)).addCallback(cb)

class DataBaseFactory:

    def __init__(self):
        self.has_type = {}
        self.objects = {}
        try:
            from db_mysql import MySQLDataBase
            self.has_type['mysql'] = True
            self.objects['mysql'] = MySQLDataBase
        except ImportError, e:
            self.has_type['mysql'] = False

        try:
            from db_pgsql import PostgreSQLDataBase
            self.has_type['pgsql'] = True
            self.objects['pgsql'] = PostgreSQLDataBase
        except ImportError, e:
            self.has_type['pgsql'] = False

    def create(self, conf, logger):
        try:
            dbtype = conf['dbtype']
            db_object = self.objects[ dbtype ]
        except KeyError, e:
            raise DatabaseError(tr('Unsupported database type'))

        return db_object(conf, logger)
