# -*- coding: utf-8 -*-

"""
Copyright(C) 2007-2008 INL
Written by Romain Bignon <romain AT inl.fr>
           Pierre Chifflier <chifflier AT inl.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

$Id: db_mysql.py 17752 2009-02-03 08:56:18Z romain $
"""

# NEEDS python-mysqldb
import _mysql_exceptions
import struct

from database import GenericDataBase
from request import Request, TriggerRequest, Ulogd2Request

#############################################
#                                           #
#            Database Object                #
#                                           #
#############################################
class MySQLDataBase(GenericDataBase):

    types = {'ulog': Request,
             'triggers': TriggerRequest,
             'ulogd2': Ulogd2Request
            }

    dbtype = "mysql"
    dbapi = "MySQLdb"
    kw_max_limit = "18446744073709551615"

    dbkw = { 'host' : 'host', 'db': 'db', 'user' : 'user', 'pass': 'passwd' }

    error_type = _mysql_exceptions.OperationalError

    def __init__(self, conf, logger):
        """
            Build DataBase object.
        """

        GenericDataBase.__init__(self, conf)

    def whereIP(self, args, key, ip):
        if self.ip_type == 6:
            if ip.prefixlen() < 128:
                if (ip.prefixlen() % 8) == 0:
                    if 0 < ip.prefixlen():
                        mask = '%032X' % ip.int()
                        mask = mask[:ip.prefixlen()/4]
                        return "%s LIKE CONCAT(0x%s, '%%')" % (key, mask)
                    else:
                        return '1'
                else:
                    first = ip.int()
                    last = ip.broadcast().int()
                    return '%s >= 0x%032X AND %s <= 0x%032X' % (key, first, key, last)
            else:
                return '%s = LPAD(0x%X, 16, 0x00)' % (key, ip.int())
        else:
            if ip.prefixlen() < 32:
                first = ip.int()
                last = ip.broadcast().int()
                return '(%s >= %s AND %s <= %s)' % (key, first, key, last)
            else:
                return '%s = %s' % (key, ip.int())

    def whereIP_REVERSE(self, args, key, ip):
        if self.ip_type == 6:
            return self._arg_where_ip(key, ip)
        else:
            value = struct.unpack("<I", struct.pack(">I", long(ip.int())))[0]
            return '%s = %s' % (key, value)
