# -*- coding: utf-8 -*-

"""
Copyright(C) 2007-2008 INL
Written by Pierre Chifflier <chifflier AT inl.fr>
           Romain Bignon <romain AT inl.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

$Id: db_pgsql.py 17754 2009-02-03 09:09:00Z romain $
"""

# NEEDS python-mysqldb
from twisted.enterprise.adbapi import safe
import psycopg2

from database import GenericDataBase
from request import PGTriggerRequest, PGRequest, PGUlogd2Request

#############################################
#                                           #
#            Database Object                #
#                                           #
#############################################
class PostgreSQLDataBase(GenericDataBase):

    types = {'ulog': PGRequest,
             'triggers': PGTriggerRequest,
             'ulogd2': PGUlogd2Request
            }

    dbtype = "pgsql"
    dbapi = "psycopg2"
    kw_max_limit = "ALL"

    dbkw = { 'host' : 'host', 'db': 'database', 'user' : 'user', 'pass': 'password' }

    error_type = psycopg2.OperationalError

    def __init__(self, conf, logger):
        """
            Build DataBase object.
        """

        GenericDataBase.__init__(self, conf)

    def whereIP(self, args, key, ip):

        prefix_lens = {4: 32, 6: 128}
        if ip.prefixlen() < prefix_lens[ip.version()]:
            return "inet '%s' >> %s" % (str(ip), key)
        else:
            return "%s = inet '%s'" % (key, str(ip))

    def whereIP_REVERSE(self, args, key, ip):
        return self.whereIP(args, key, ip)
