# -*- coding: utf-8 -*-

"""
Copyright(C) 2007-2008 INL
Written by Romain Bignon <romain AT inl.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

$Id: edenwall.py 17064 2008-12-01 13:57:01Z romain $
"""

from pages import BasePage
from nevow import loaders, tags, inevow, rend
from nevow.i18n import _, render as i18nrender
from frags import FragmentFactory
from tools import trans
import re
import time
import os
try:
    from nucentral.client import Service
except:
    from wrapper.client import Service

mypath = os.path.dirname(__file__) + os.sep

def get_uptime():

     try:
         f = open( "/proc/uptime" )
         contents = f.read().split()
         f.close()
     except:
        return "Cannot open uptime file: /proc/uptime"

     total_seconds = float(contents[0])

     # Helper vars:
     MINUTE  = 60
     HOUR    = MINUTE * 60
     DAY     = HOUR * 24

     # Get the days, hours, etc:
     days    = int( total_seconds / DAY )

     return days

class TrafficFragment(rend.Fragment):

    docFactory = loaders.xmlfile(mypath + 'xml/box.xml')

    def render_title(self, ctx, data):

        return _('Internet traffic')

    def render_icon(self, ctx, data):

        return ''

    def render_content(self, ctx, data):
        return tags.img(src='/cgi-bin/systemgraph.cgi?4h-net-eth0&ypoints-100&xpoints-300&nolegend')

class DecisionFragment(rend.Fragment):

    docFactory = loaders.xmlfile(mypath + 'xml/box.xml')

    def render_title(self, ctx, data):

        return _('Firewall Decisions')

    def render_icon(self, ctx, data):

        return ''

    def render_content(self, ctx, data):
        return tags.img(src='/cgi-bin/systemgraph.cgi?4h-nuauthsql-Connection_stats&ypoints-100&xpoints-300&nolegend')

class EdenIndexPage(BasePage):

    render_i18n = i18nrender()

    docFactory = loaders.xmlfile(mypath + 'xml/edenindex.xml')

    addSlash = False

    def render_slots(self, ctx, data):

        ctx.fillSlots('PATH', self.conf.get('Links', 'url'))
        return ctx.tag

    def render_edencss(self, ctx, data):
        if int(self.conf.get('Misc', 'edenwall_integration')) > 0:
            return loaders.xmlfile(mypath + 'xml/edencss.xml')

        return ''

    def render_edennavbar(self, ctx, data):
        if int(self.conf.get('Misc', 'edenwall_integration')) > 0:
            from nulog import EdenNavBar
            return ctx.tag[EdenNavBar(is_index=True)]
        return ''

    def _float_render(self, value):
        """
            Default is %1.4f, and we use this callback
            to only show %1.2f.
        """

        if isinstance(value, (list, tuple)):
            return '%1.2f' % value[0][0]
        else:
            return '0'

    def _must_int(self, result):

        try:
            return result[0]
        except:
            return 0

    def render_average1(self, ctx, data):
        return Service.callService("nulog-core", "count", ('average1')).addCallback(self._float_render)

    def render_average5(self, ctx, data):
        return Service.callService("nulog-core", "count", ('average5')).addCallback(self._float_render)

    def render_average15(self, ctx, data):
        return Service.callService("nulog-core", "count", ('average15')).addCallback(self._float_render)

    def render_userconnected(self, ctx, data):
        return Service.callService("nulog-core", "count", ('conusers')).addCallback(self._must_int)


    def render_uptime(self, ctx, data):

        now = time.localtime()
        uptime = get_uptime()

        return [_('Uptime: %d days') % uptime, tags.br(),
                _('Time: %s') % time.strftime('%H:%M:%S', now)]

    def render_load(self, ctx, data):
        child = os.popen('/usr/bin/uptime')
        line = child.read()
        child.close()

        m = re.match(".*(\d+\.\d+)$", line)
        load = line
        if m:
            load = m.group(1)

        child = os.popen("/usr/bin/rrdtool fetch /var/lib/rrd/net.rrd.eth0 AVERAGE -r 60 -s -90s | grep -v nan | grep -v '^$' |grep -v outerrors | head -n1 | awk '{print $2; print $3;}'")
        nm = child.read().split('\n')

        try:
            unit = ["", "k", "M", "G", "T"]
            b_in = float(nm[0])
            b_out = float(nm[1])

            i = 0
            while b_in >= 1024:
                b_in /= 1024
                i += 1
            b_in = '%.3f %sb/s' % (b_in, unit[i])

            i = 0
            while b_out >= 1024:
                b_out /= 1024
                i += 1
            b_out = '%.3f %sb/s' % (b_out, unit[i])

        except:
            b_in = '?'
            b_out = '?'

        child.close()

        return [ tags.div(_class="perfs")[_('%s of system load') % load],
                tags.div(_class="perfs") [
                _('bandwidth (in): %s') % b_in, tags.br(),
                _('bandwidth (out): %s') % b_out
                ]]

    def render_traffic(self, ctx, data):

        return tags.div(_id='traffic', _class='frag')[TrafficFragment()]

    def render_decision(self, ctx, data):

        return tags.div(_id='decision', _class='frag')[DecisionFragment()]

    def render_apps(self, ctx, data):
        return FragmentFactory(ctx, 'pie', 'EdenIndex_Apps', 'Dropped applications:',
                               self.conf.get('Links', 'url'), {}, icon='',
                               switch=True, divid=True, save=False)

    def render_users(self, ctx, data):
        return FragmentFactory(ctx, 'pie', 'EdenIndex_UserTable', 'Per-user dropped packets:',
                               self.conf.get('Links', 'url'), {}, icon='',
                               switch=True, divid=True, save=False)

    def render_connected(self, ctx, data):
        return FragmentFactory(ctx, 'table', 'EdenConUserTable', 'Connected',
                               self.conf.get('Links', 'url'), {}, icon='',
                               switch=False, divid=True, save=False)

    def render_nulogjsi18n(self, ctx, data):
        from nulog import NulogJsI18NFragment
        frag = NulogJsI18NFragment()
        return ctx.tag[frag]

